/*
 * Copyright (c) 2005 CNRS/LAAS
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */
#include <portLib.h>
#include <errno.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include <linux/joystick.h>

#include "joystick.h"
#include "joystickStruct.h"

#define NAME_LENGTH 256
/*----------------------------------------------------------------------*/
/**
 ** Init joystick 
 **/
STATUS joystickInit(char *dev, JOYSTICK_DATA* joystick)
{
  int joy;
  int version;
  unsigned char axes, buttons;
  char name[NAME_LENGTH];
	
  joy = open(dev, O_RDONLY);
  if (joy < 0) {
    perror(dev);
    return ERROR;
  }
  ioctl(joy, JSIOCGVERSION, &version);
  ioctl(joy, JSIOCGAXES, &axes);
  ioctl(joy, JSIOCGBUTTONS, &buttons);
  ioctl(joy, JSIOCGNAME(NAME_LENGTH), name);
	
  printf("Joystick (%s) has %d axes and %d buttons.\n"
	 "\tDriver version is %d.%d.%d.\n",
	 name, axes, buttons, version >> 16, 
	 (version >> 8) & 0xff, version & 0xff);
  if (fcntl(joy, F_SETFL, O_NONBLOCK) < 0) {
    fprintf(stderr, "error setting non blocking mode on %s: %s\n", dev, strerror(errno));
    close(joy);
    free(joystick);
    return ERROR;
  }
  if( ((int)axes > JOYSTICK_N_AXES) || ((int)buttons > JOYSTICK_N_BUTTONS)) {
    fprintf(stderr, "error too many axis or buttons on %s: %s\n", dev, strerror(errno));
    return ERROR;
  }
  
  joystick->fd = joy;
  return OK;
}

/*----------------------------------------------------------------------*/

void joystickEnd(JOYSTICK_DATA *joystick)
{
  close(joystick->fd);
  free(joystick);
}

/*----------------------------------------------------------------------*/

int joystickPoll(JOYSTICK_DATA *joystick)
{
  struct js_event js;
  ssize_t n;
  int ret = 0;

  do {
    n = read(joystick->fd, &js, sizeof(struct js_event));
    if (n == -1) {
      if (errno == EAGAIN)
	return ret;
      else {
	printf("joystickPoll: %s\n", strerror(errno));
	return -1;
      }
    } else if (n == 0) {
      return ret;
    } else if (n != sizeof(struct js_event)) {
      /* short read */
      return ret;
    }
    /* n == sizeof(struct js_event) */
    switch (js.type & ~JS_EVENT_INIT) {
    case JS_EVENT_BUTTON:
      joystick->buttons[js.number] = js.value;
      break;
    case JS_EVENT_AXIS:
      joystick->axes[js.number] = js.value;
      break;
    } /* switch */
    ret++;
  } while (1);
}

/*----------------------------------------------------------------------*/
		
		

	      
